@@ -14,6 +14,10 @@ module Sequel
           opts[:password]
         )
       end
+
+      def disconnect_connection(c)
+        c.disconnect
+      end
       
       def execute(sql, opts={})
         synchronize(opts[:server]) do |conn|
@@ -23,16 +27,10 @@ module Sequel
         end
       end
       alias_method :do, :execute
-
-      private
-
-      def disconnect_connection(c)
-        c.disconnect
-      end
     end
     
     class Dataset < Sequel::Dataset
-      SELECT_CLAUSE_METHODS = clause_methods(:select, %w'distinct columns from join where group having compounds order limit')
+      SELECT_CLAUSE_METHODS = clause_methods(:select, %w'select distinct columns from join where group having compounds order limit')
 
       Database::DatasetClass = self
       
