@@ -20,12 +20,19 @@
 # for both PGRange and Range that use the standard Sequel literalization
 # callbacks, so they work on all adapters.
 #
-# To turn an existing Range into a PGRange:
+# To turn an existing Range into a PGRange, use Sequel.pg_range:
+#
+#   Sequel.pg_range(range)
+#
+# If you have loaded the {core_extensions extension}[link:files/doc/core_extensions_rdoc.html]),
+# or you have loaded the {core_refinements extension}[link:files/doc/core_refinements_rdoc.html])
+# and have activated refinements for the file, you can also use Range#pg_range:
 #
 #   range.pg_range 
 #
 # You may want to specify a specific range type:
 #
+#   Sequel.pg_range(range, :daterange)
 #   range.pg_range(:daterange)
 #
 # If you specify the range database type, Sequel will automatically cast
@@ -53,6 +60,7 @@ Sequel.require 'adapters/utils/pg_types'
 module Sequel
   module Postgres
     class PGRange
+      include Sequel::SQL::AliasMethods
 
       # Map of string database type names to type symbols (e.g. 'int4range' => :int4range),
       # used in the schema parsing.
@@ -177,7 +185,6 @@ module Sequel
         # Reset the conversion procs if using the native postgres adapter,
         # and extend the datasets to correctly literalize ruby Range values.
         def self.extended(db)
-          db.reset_conversion_procs if db.respond_to?(:reset_conversion_procs)
           db.extend_datasets(DatasetMethods)
         end
 
@@ -216,12 +223,11 @@ module Sequel
         # Manually override the typecasting for tsrange and tstzrange types so that
         # they use the database's timezone instead of the global Sequel
         # timezone.
-        def get_conversion_procs(conn)
+        def get_conversion_procs
           procs = super
 
-          converter = method(:to_application_timestamp)
-          procs[3908] = Parser.new("tsrange", converter)
-          procs[3910] = Parser.new("tstzrange", converter)
+          procs[3908] = Parser.new("tsrange", procs[1114])
+          procs[3910] = Parser.new("tstzrange", procs[1184])
           if defined?(PGArray::Creator)
             procs[3909] = PGArray::Creator.new("tsrange", procs[3908])
             procs[3911] = PGArray::Creator.new("tstzrange", procs[3910])
@@ -475,13 +481,44 @@ module Sequel
     end
   end
 
+  module SQL::Builders
+    # Convert the object to a Postgres::PGRange.
+    def pg_range(v, db_type=nil)
+      case v
+      when Postgres::PGRange
+        if db_type.nil? || v.db_type == db_type
+          v
+        else
+          Postgres::PGRange.new(v.begin, v.end, :exclude_begin=>v.exclude_begin?, :exclude_end=>v.exclude_end?, :db_type=>db_type)
+        end
+      when Range
+        Postgres::PGRange.from_range(v, db_type)
+      else
+        # May not be defined unless the pg_range_ops extension is used
+        pg_range_op(v)
+      end
+    end
+  end
+
   Database.register_extension(:pg_range, Postgres::PGRange::DatabaseMethods)
 end
 
-class Range 
-  # Create a new PGRange using the receiver as the input range,
-  # with the given database type.
-  def pg_range(db_type=nil)
-    Sequel::Postgres::PGRange.from_range(self, db_type)
+if Sequel.core_extensions?
+  class Range 
+    # Create a new PGRange using the receiver as the input range,
+    # with the given database type.
+    def pg_range(db_type=nil)
+      Sequel::Postgres::PGRange.from_range(self, db_type)
+    end
+  end
+end
+
+if defined?(Sequel::CoreRefinements)
+  module Sequel::CoreRefinements
+    refine Range do
+      def pg_range(db_type=nil)
+        Sequel::Postgres::PGRange.from_range(self, db_type)
+      end
+    end
   end
 end
