@@ -20,35 +20,35 @@ describe "Sequel::Model()" do
   end
 
   it "should return a model subclass with a dataset with the default database and given table name if given a LiteralString" do
-    c = Sequel::Model('blah'.lit)
+    c = Sequel::Model(Sequel.lit('blah'))
     c.superclass.should == Sequel::Model
     c.db.should == @db
-    c.table_name.should == 'blah'.lit
+    c.table_name.should == Sequel.lit('blah')
   end
 
   it "should return a model subclass with a dataset with the default database and given table name if given an SQL::Identifier" do
-    c = Sequel::Model(:blah.identifier)
+    c = Sequel::Model(Sequel.identifier(:blah))
     c.superclass.should == Sequel::Model
     c.db.should == @db
-    c.table_name.should == :blah.identifier
+    c.table_name.should == Sequel.identifier(:blah)
   end
 
   it "should return a model subclass with a dataset with the default database and given table name if given an SQL::QualifiedIdentifier" do
-    c = Sequel::Model(:blah.qualify(:boo))
+    c = Sequel::Model(Sequel.qualify(:boo, :blah))
     c.superclass.should == Sequel::Model
     c.db.should == @db
-    c.table_name.should == :blah.qualify(:boo)
+    c.table_name.should == Sequel.qualify(:boo, :blah)
   end
 
   it "should return a model subclass with a dataset with the default database and given table name if given an SQL::AliasedExpression" do
-    c = Sequel::Model(:blah.as(:boo))
+    c = Sequel::Model(Sequel.as(:blah, :boo))
     c.superclass.should == Sequel::Model
     c.db.should == @db
     c.table_name.should == :boo
   end
 
   it "should return a model subclass with the given dataset if given a dataset using an SQL::Identifier" do
-    ds = @db[:blah.identifier]
+    ds = @db[Sequel.identifier(:blah)]
     c = Sequel::Model(ds)
     c.superclass.should == Sequel::Model
     c.dataset.should == ds
@@ -84,29 +84,29 @@ describe "Sequel::Model()" do
 
     it "should work without raising an exception with an SQL::Identifier " do
       proc do
-        class ::Album < Sequel::Model(:table.identifier); end
-        class ::Album < Sequel::Model(:table.identifier); end
+        class ::Album < Sequel::Model(Sequel.identifier(:table)); end
+        class ::Album < Sequel::Model(Sequel.identifier(:table)); end
       end.should_not raise_error
     end
 
     it "should work without raising an exception with an SQL::QualifiedIdentifier " do
       proc do
-        class ::Album < Sequel::Model(:table.qualify(:schema)); end
-        class ::Album < Sequel::Model(:table.qualify(:schema)); end
+        class ::Album < Sequel::Model(Sequel.qualify(:schema, :table)); end
+        class ::Album < Sequel::Model(Sequel.qualify(:schema, :table)); end
       end.should_not raise_error
     end
 
     it "should work without raising an exception with an SQL::AliasedExpression" do
       proc do
-        class ::Album < Sequel::Model(:table.as(:alias)); end
-        class ::Album < Sequel::Model(:table.as(:alias)); end
+        class ::Album < Sequel::Model(Sequel.as(:table, :alias)); end
+        class ::Album < Sequel::Model(Sequel.as(:table, :alias)); end
       end.should_not raise_error
     end
 
     it "should work without raising an exception with an LiteralString" do
       proc do
-        class ::Album < Sequel::Model('table'.lit); end
-        class ::Album < Sequel::Model('table'.lit); end
+        class ::Album < Sequel::Model(Sequel.lit('table')); end
+        class ::Album < Sequel::Model(Sequel.lit('table')); end
       end.should_not raise_error
     end
 
@@ -126,16 +126,16 @@ describe "Sequel::Model()" do
 
     it "should work without raising an exception with a dataset with an SQL::Identifier" do
       proc do
-        class ::Album < Sequel::Model(@db[:table.identifier]); end
-        class ::Album < Sequel::Model(@db[:table.identifier]); end
+        class ::Album < Sequel::Model(@db[Sequel.identifier(:table)]); end
+        class ::Album < Sequel::Model(@db[Sequel.identifier(:table)]); end
       end.should_not raise_error
     end
 
     it "should raise an exception if anonymous model caching is disabled" do
       Sequel::Model.cache_anonymous_models = false
       proc do
-        class ::Album < Sequel::Model(@db[:table.identifier]); end
-        class ::Album < Sequel::Model(@db[:table.identifier]); end
+        class ::Album < Sequel::Model(@db[Sequel.identifier(:table)]); end
+        class ::Album < Sequel::Model(@db[Sequel.identifier(:table)]); end
       end.should raise_error
     end
   end
@@ -195,25 +195,25 @@ describe Sequel::Model, "dataset & schema" do
 
   it "set_dataset should take a LiteralString" do
     @model.db = MODEL_DB
-    @model.set_dataset('foo'.lit)
-    @model.table_name.should == 'foo'.lit
+    @model.set_dataset(Sequel.lit('foo'))
+    @model.table_name.should == Sequel.lit('foo')
   end
 
   it "set_dataset should take an SQL::Identifier" do
     @model.db = MODEL_DB
-    @model.set_dataset(:foo.identifier)
-    @model.table_name.should == :foo.identifier
+    @model.set_dataset(Sequel.identifier(:foo))
+    @model.table_name.should == Sequel.identifier(:foo)
   end
 
   it "set_dataset should take an SQL::QualifiedIdentifier" do
     @model.db = MODEL_DB
-    @model.set_dataset(:foo.qualify(:bar))
-    @model.table_name.should == :foo.qualify(:bar)
+    @model.set_dataset(Sequel.qualify(:bar, :foo))
+    @model.table_name.should == Sequel.qualify(:bar, :foo)
   end
 
   it "set_dataset should take an SQL::AliasedExpression" do
     @model.db = MODEL_DB
-    @model.set_dataset(:foo.as(:bar))
+    @model.set_dataset(Sequel.as(:foo, :bar))
     @model.table_name.should == :bar
   end
 
@@ -248,22 +248,22 @@ describe Sequel::Model, "dataset & schema" do
   end
 
   it "should raise an error on set_dataset if there is an error connecting to the database" do
-    @model.meta_def(:columns){raise Sequel::DatabaseConnectionError}
+    def @model.columns() raise Sequel::DatabaseConnectionError end
     proc{@model.set_dataset(Sequel::Database.new[:foo].join(:blah))}.should raise_error
   end
 
   it "should not raise an error if there is a problem getting the columns for a dataset" do
-    @model.meta_def(:columns){raise Sequel::Error}
+    def @model.columns() raise Sequel::Error end
     proc{@model.set_dataset(MODEL_DB[:foo].join(:blah))}.should_not raise_error
   end
 
   it "doesn't raise an error on set_dataset if there is an error raised getting the schema" do
-    @model.meta_def(:get_db_schema){raise Sequel::Error}
+    def @model.get_db_schema(*) raise Sequel::Error end
     proc{@model.set_dataset(MODEL_DB[:foo])}.should_not raise_error
   end
 
   it "doesn't raise an error on inherited if there is an error setting the dataset" do
-    @model.meta_def(:set_dataset){raise Sequel::Error}
+    def @model.set_dataset(*) raise Sequel::Error end
     proc{Class.new(@model)}.should_not raise_error
   end
 end
@@ -361,12 +361,12 @@ describe Sequel::Model, ".subset" do
   specify "should create a filter on the underlying dataset" do
     proc {@c.new_only}.should raise_error(NoMethodError)
     
-    @c.subset(:new_only) {:age.sql_number < 'new'}
+    @c.subset(:new_only){age < 'new'}
     
     @c.new_only.sql.should == "SELECT * FROM items WHERE (age < 'new')"
     @c.dataset.new_only.sql.should == "SELECT * FROM items WHERE (age < 'new')"
     
-    @c.subset(:pricey) {:price.sql_number > 100}
+    @c.subset(:pricey){price > 100}
     
     @c.pricey.sql.should == "SELECT * FROM items WHERE (price > 100)"
     @c.dataset.pricey.sql.should == "SELECT * FROM items WHERE (price > 100)"
@@ -376,7 +376,7 @@ describe Sequel::Model, ".subset" do
   end
 
   specify "should not override existing model methods" do
-    @c.meta_def(:active){true}
+    def @c.active() true end
     @c.subset(:active, :active)
     @c.active.should == true
   end
@@ -393,15 +393,15 @@ describe Sequel::Model, ".find" do
     @c.find(:name => 'sharon').should be_a_kind_of(@c)
     MODEL_DB.sqls.should == ["SELECT * FROM items WHERE (name = 'sharon') LIMIT 1"]
 
-    @c.find(:name.like('abc%')).should be_a_kind_of(@c)
+    @c.find(Sequel.expr(:name).like('abc%')).should be_a_kind_of(@c)
     MODEL_DB.sqls.should == ["SELECT * FROM items WHERE (name LIKE 'abc%') LIMIT 1"]
   end
   
   specify "should accept filter blocks" do
-    @c.find{:id.sql_number > 1}.should be_a_kind_of(@c)
+    @c.find{id > 1}.should be_a_kind_of(@c)
     MODEL_DB.sqls.should == ["SELECT * FROM items WHERE (id > 1) LIMIT 1"]
 
-    @c.find {(:x.sql_number > 1) & (:y.sql_number < 2)}.should be_a_kind_of(@c)
+    @c.find{(x > 1) & (y < 2)}.should be_a_kind_of(@c)
     MODEL_DB.sqls.should == ["SELECT * FROM items WHERE ((x > 1) AND (y < 2)) LIMIT 1"]
   end
 end
@@ -652,7 +652,7 @@ describe "Model.db_schema" do
   specify "should automatically set a singular primary key based on the schema" do
     ds = @dataset
     d = ds.db
-    d.meta_def(:schema){|table, *opts| [[:x, {:primary_key=>true}]]}
+    def d.schema(table, *opts) [[:x, {:primary_key=>true}]] end
     @c.primary_key.should == :id
     @c.dataset = ds
     @c.db_schema.should == {:x=>{:primary_key=>true}}
@@ -662,7 +662,7 @@ describe "Model.db_schema" do
   specify "should automatically set the composite primary key based on the schema" do
     ds = @dataset
     d = ds.db
-    d.meta_def(:schema){|table, *opts| [[:x, {:primary_key=>true}], [:y, {:primary_key=>true}]]}
+    def d.schema(table, *opts) [[:x, {:primary_key=>true}], [:y, {:primary_key=>true}]] end
     @c.primary_key.should == :id
     @c.dataset = ds
     @c.db_schema.should == {:x=>{:primary_key=>true}, :y=>{:primary_key=>true}}
@@ -672,7 +672,7 @@ describe "Model.db_schema" do
   specify "should automatically set no primary key based on the schema" do
     ds = @dataset
     d = ds.db
-    d.meta_def(:schema){|table, *opts| [[:x, {:primary_key=>false}], [:y, {:primary_key=>false}]]}
+    def d.schema(table, *opts) [[:x, {:primary_key=>false}], [:y, {:primary_key=>false}]] end
     @c.primary_key.should == :id
     @c.dataset = ds
     @c.db_schema.should == {:x=>{:primary_key=>false}, :y=>{:primary_key=>false}}
@@ -682,7 +682,7 @@ describe "Model.db_schema" do
   specify "should not modify the primary key unless all column schema hashes have a :primary_key entry" do
     ds = @dataset
     d = ds.db
-    d.meta_def(:schema){|table, *opts| [[:x, {:primary_key=>false}], [:y, {}]]}
+    def d.schema(table, *opts) [[:x, {:primary_key=>false}], [:y, {}]] end
     @c.primary_key.should == :id
     @c.dataset = ds
     @c.db_schema.should == {:x=>{:primary_key=>false}, :y=>{}}
